# Skript to plot UpSetR plots
# Input are tables generated by Aizhan and Ulli 
# https://docs.google.com/spreadsheets/d/11Du-dlLpSjDEKtA9O-ClR1QFXNm0UTFgB1nwcTDbpo8/edit#gid=1973046150
# Antibody Names have been adapted to contain no spaces & /
# Yes and No has been turned to 0 and 1

library(UpSetR)
library(ggplot2)

input <- read.table("PBMC_97ABs_Venn.tab", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
png("PBMC_97ABs_Venn", width = 700, height = 400, res = 300, units="mm")
upset(data=input, nsets=100, nintersects=20, order.by=c("freq"), 
      #intersections = list("X0", "X1", "X2", "X3", "X4", "X5", "X6", "X7"),
      matrix.color = 3,        # color of dots and lines
      shade.color = 3,        # shading in table
      shade.alpha = 0.25,        # Transparency of shading in matrix, 0 is no shading
      keep.order = FALSE,
      main.bar.color = "grey20",
      line.size = 0.5,        # lines connecting dots
      point.size = 5,
      mainbar.y.label = "Number of antibodies",
      #mainbar.y.max = "100" #?
      sets.x.label = "Number of confirmed antibodies in sample ",        # label of sets barplots
      #att.color = 5,    #?
      #group.by = "degree", #?
      matrix.dot.alpha = 1,        # transparency of dots not in set, 1 is strongest
      sets.bar.color = "grey38",
      mb.ratio = c(0.7, 0.3),        # ratio between main bar plot and matrix plot
      text.scale = c(5,3,2,2,3,3)
      # Can be a universal scale, or a vector containing individual scales in the following format: 
      # c(intersection size title, intersection size tick labels, set size title, set size tick labels, set names, numbers above bars)
)
dev.off()

input <- read.table("PBMC_additional16ABs_Venn.tab", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
png("PBMC_additional16ABs_Venn", width = 700, height = 400, res = 300, units="mm")
upset(data=input, nsets=100, nintersects=20, order.by=c("freq"), 
      #intersections = list("X0", "X1", "X2", "X3", "X4", "X5", "X6", "X7"),
      matrix.color = 3,        # color of dots and lines
      shade.color = 3,        # shading in table
      shade.alpha = 0.25,        # Transparency of shading in matrix, 0 is no shading
      keep.order = FALSE,
      main.bar.color = "grey20",
      line.size = 0.5,        # lines connecting dots
      point.size = 5,
      mainbar.y.label = "Number of antibodies",
      #mainbar.y.max = "100" #?
      sets.x.label = "Number of confirmed antibodies in sample ",        # label of sets barplots
      #att.color = 5,    #?
      #group.by = "degree", #?
      matrix.dot.alpha = 1,        # transparency of dots not in set, 1 is strongest
      sets.bar.color = "grey38",
      mb.ratio = c(0.7, 0.3),        # ratio between main bar plot and matrix plot
      text.scale = c(5,3,2,2,3,3)
      # Can be a universal scale, or a vector containing individual scales in the following format: 
      # c(intersection size title, intersection size tick labels, set size title, set size tick labels, set names, numbers above bars)
)
dev.off()

input <- read.table("PBMC_healthy_dissociation_97ABS_Venn.tab", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
png("PBMC_healthy_dissociation_97ABS_Venn", width = 700, height = 400, res = 300, units="mm")

upset(data=input, nsets=100, nintersects=20, order.by=c("freq"), 
      #intersections = list("X0", "X1", "X2", "X3", "X4", "X5", "X6", "X7"),
      matrix.color = 3,        # color of dots and lines
      shade.color = 3,        # shading in table
      shade.alpha = 0.25,        # Transparency of shading in matrix, 0 is no shading
      keep.order = FALSE,
      main.bar.color = "grey20",
      line.size = 0.5,        # lines connecting dots
      point.size = 5,
      mainbar.y.label = "Number of antibodies",
      #mainbar.y.max = "100" #?
      sets.x.label = "Number of confirmed antibodies in sample ",        # label of sets barplots
      #att.color = 5,    #?
      #group.by = "degree", #?
      matrix.dot.alpha = 1,        # transparency of dots not in set, 1 is strongest
      sets.bar.color = "grey38",
      mb.ratio = c(0.7, 0.3),        # ratio between main bar plot and matrix plot
      text.scale = c(5,3,2,2,3,3)
      # Can be a universal scale, or a vector containing individual scales in the following format: 
      # c(intersection size title, intersection size tick labels, set size title, set size tick labels, set names, numbers above bars)
)
dev.off()

