# Script to Analyse ADT Data in combination with GEX.
# ToDo - Modularize Script (aka. ADT only analysis could be done in a seperate script.
# ToDo - Potentially split table generation & plotting.
# ToDo - Change Bar  Plots to Ridgeplots


## Example Call
#Rscript analyseADT.R --analysis=/cluster/project/nexus/phrt/levesque_melanoma_2019/data/20191130/PBMC_mix/analysis_PBMC003/ --analysisADT=/cluster/project/nexus/phrt/levesque_melanoma_2019/data/20191130/PBMC_mix/analysis_pooledSample//CellRangerADT/PBMCmix_pooled/outs/filtered_feature_bc_matrix/ --config=/cluster/project/nexus/phrt/levesque_melanoma_2019/data/20191130/PBMC_mix/snake_analysis_files_PBMC003/config_scTranscriptomics.json --lookup=/cluster/project/nexus/linda/git_eth/levesque_melanoma_2019/metadata/lookup_gene_protein.20191130.txt --sampleName=PBMC003 --threshold=0
 


library(cowplot)
library(ggridges)
library(optparse)
library(Seurat)
library(gtable)
library(rjson)
library(plyr)
library(tibble)
library(dplyr)
library(reshape)
library(ggplot2)
library(limma)
library(viridis)
library(grDevices)

option_list = list(
make_option("--analysis", type = "character", help = "Path to the main analysis folder containing results of GEX pipline."),
make_option("--analysisADT", type = "character", help = "Path to the pooled ADT analysis folder (folder that contains feature, barcode and matrix file)."),
make_option("--config", type = "character", help = "Path to the config file used in the GEX pipeline."),
make_option("--lookup", type = "character", help = "Path to a lookup table for gene & protein names."),
make_option("--threshold", type = "integer", help = "MinUMI count to include antibody as valid."),
make_option("--sampleName", type = "character", help = "SampleName")
)
opt_parser = OptionParser(option_list = option_list)
opt = parse_args(opt_parser)

outfolder <- paste(opt$analysis,"/ADTAnalysis/",sep="")
dir.create(outfolder, showWarnings=FALSE)
outprefix <- paste(outfolder, opt$sampleName, sep="")

# define function
f.round.preserve.sum = function(x, digits = 0) {
  up = 10 ^ digits
  x = x * up
  y = floor(x)
  indices = tail(order(x-y), round(sum(x)) - sum(y))
  y[indices] = y[indices] + 1
  y / up
}

# set general options/parameters
options(stringsAsFactors = FALSE)

####
# Read in files 
RDSfolder <- paste(opt$analysis,"/atypical_removed/",sep="")
RDSfile <- list.files( RDSfolder, pattern="\\.RDS$")
if (length(RDSfile)>1){
print(paste("Only one RDS file is expected in the following folder: ", RDSfolder, sep=""))
}
my_sce <- readRDS(paste(RDSfolder,RDSfile, sep=""))


CellRangerADTfolder <- opt$analysisADT
CellRangerADT <- Read10X(data.dir = CellRangerADTfolder) 

###
# Ridgeplot ABs
# In the current experimental setting it makes sense to plot them on the pooled samples.
# If this is also the case in the future, it might be preferable to have an extra script for that, since the rest of the script considers only the individual samples.
###
print(paste(outprefix,"ridgeplot.png",sep="."))
#rigdePlot = RidgePlot(seurat_object, features=c("CD2"), assay = "ADT", ncol=2)
#ggplot2::ggsave(filename= paste(outprefix,"ridgeplot.png",sep="."),plot = rigdePlot)
dir.create(paste(outfolder, "/RidgeplotsPooledSample/", sep=""))
ridgeout=paste(outfolder, "/RidgeplotsPooledSample/", sep="")
plot_list = list()
for (i in seq(1,length(rownames(CellRangerADT)))){
        #tryCatch(FetchData(seurat_object, vars=rownames(CellRangerADT)[i]), error = function(err){ next})
        test <- as.matrix(CellRangerADT[i,])
        colnames(test) <- rownames(CellRangerADT)[i]
        test <- melt(test)
        colnames(test) <- c("barcode","antibody","value")
        test$value[test$value == 0] <- NA
        test$log <- log(test$value)
        print(head(test))
        s <-ggplot(test, aes(x = log, y = antibody , fill = antibody,height=..ndensity..)) +
          geom_density_ridges() +
          scale_x_continuous(limits = c(0,max(test$log)+1)) +
          geom_vline(xintercept = log(opt$threshold)) +
          theme_ridges() +
          theme(legend.position = "none",axis.title.y = element_blank(),plot.margin=unit(c(0.5,0.5,0.5,1),"cm"))
        plot_list[[i]]<- s
}
for (i in seq(1,15*ceiling(length(rownames(CellRangerADT))/15),15)){
        plot_sub <- plot_list[i:min(i+15,length(rownames(CellRangerADT)))]
        plot_group =  plot_grid(plotlist = plot_sub, ncol = 4)
	if (i+15 > length(rownames(CellRangerADT))){
	ggplot2::ggsave(filename= paste(paste(ridgeout,opt$sampleName,".Antibody",sep=""),i,"to",length(rownames(CellRangerADT)),"ridgeplot.png",sep="_"),width=21,height=14, plot = plot_group)
	}else{
         ggplot2::ggsave(filename= paste(paste(ridgeout,opt$sampleName,".Antibody",sep=""),i,"to",i+15,"ridgeplot.png",sep="_"),width=21,height=14, plot = plot_group)
}}

###
# Ridgeplots are plotted using unnormalized ADT Values (they are needed to improve labmethods)
# Normalize now for the remaining pipeline
###

##seurat_object <- CreateSeuratObject(CellRangerADT, assay="ADT")
#seurat_object <- NormalizeData(seurat_object, assay = "ADT", normalization.method = "CLR")
#CellRangerADT <- ScaleData(seurat_object, assay = "ADT")
#print("Test")



###
# This part is Sample Specific for the Levesque Pilot Samples. Should not be necessary later on! 
###

#rownames(x = CellRangerADT) <- gsub(pattern = "TotalSeq-C_", replacement = "", 
#                                             x = rownames(x = CellRangerADT))
#rownames(x = CellRangerADT) <- gsub(pattern = "_konjugiert", replacement = "", 
 #                                            x = rownames(x = CellRangerADT))
#rownames(CellRangerADT)
####

# Prepare Color-Config
color_config <- fromJSON(file=opt$config)$resources$colour_config
config <- read.csv(color_config, sep = "\t", stringsAsFactors = FALSE)
config$cell_type <- gsub(pattern = "([^_]+)_.*", "\\1", config$cell_type)
stopifnot(length(config$colour) == length(config$cell_type))
stopifnot(config$colour == unique(config$colour))
ct.color = c(config$colour, "grey50", "black")
names(ct.color) <- c(config$cell_type, "uncertain", "unknown")

# Read in LookUp Table:
lookup<-read.csv(opt$lookup, header=TRUE,sep="\t")
print(lookup)
lookup <- lookup[order(lookup$Antibody),]
print(lookup)
###
#Filter for identical cell barcodes in ADT & GEX analysis
print("test")
joint.bcs <- intersect(colnames(my_sce), colnames(CellRangerADT))
my_sce <- my_sce[,joint.bcs]
CellRangerADT <- CellRangerADT[,joint.bcs]

###
# Ridgeplot ABs
# In some cases it seems that Ridgeplots on the hashed sample instead of the pooled sample seems to be usefull.
# Code duplication --> TODO: Remove.
###
print(paste(outprefix,"ridgeplot.png",sep="."))
#rigdePlot = RidgePlot(seurat_object, features=c("CD2"), assay = "ADT", ncol=2)
#ggplot2::ggsave(filename= paste(outprefix,"ridgeplot.png",sep="."),plot = rigdePlot)
dir.create(paste(outfolder, "/RidgeplotsSingleSample/", sep=""))
ridgeout=paste(outfolder, "/RidgeplotsSingleSample/", sep="")
plot_list = list()
for (i in seq(1,length(rownames(CellRangerADT)))){
        #tryCatch(FetchData(seurat_object, vars=rownames(CellRangerADT)[i]), error = function(err){ next})
        test <- as.matrix(CellRangerADT[i,])
        colnames(test) <- rownames(CellRangerADT)[i]
        test <- melt(test)
        colnames(test) <- c("barcode","antibody","value")
        test$value[test$value == 0] <- NA
        test$log <- log(test$value)
        print(head(test))
        s <-ggplot(test, aes(x = log, y = antibody , fill = antibody,height=..ndensity..)) +
          geom_density_ridges() +
          scale_x_continuous(limits = c(0,max(test$log)+1)) +
          geom_vline(xintercept = log(opt$threshold)) +
          theme_ridges() +
          theme(legend.position = "none",axis.title.y = element_blank(),plot.margin=unit(c(0.5,0.5,0.5,1),"cm"))
        plot_list[[i]]<- s
}
for (i in seq(1,15*ceiling(length(rownames(CellRangerADT))/15),15)){
        plot_sub <- plot_list[i:min(i+15,length(rownames(CellRangerADT)))]
        plot_group =  plot_grid(plotlist = plot_sub, ncol = 4)
        if (i+15 > length(rownames(CellRangerADT))){
        ggplot2::ggsave(filename= paste(paste(ridgeout,opt$sampleName,".Antibody",sep=""),i,"to",length(rownames(CellRangerADT)),"ridgeplot.png",sep="_"),width=21,height=14, plot = plot_group)
        }else{
         ggplot2::ggsave(filename= paste(paste(ridgeout,opt$sampleName,".Antibody",sep=""),i,"to",i+15,"ridgeplot.png",sep="_"),width=21,height=14, plot = plot_group)
}}


###
# Create a table containing phenograph_clusters, antibodies and celltypes per cellbarcode.
print("Create a table containing phenograph_clusters, antibodies and celltypes per cellbarcode.")
antibodies<-rownames(CellRangerADT)
barcodes <- colnames(CellRangerADT)
df_colData_cells <- as.data.frame(colData(my_sce), stringsAsFactors = FALSE)
table1 <- select(df_colData_cells, phenograph_clusters,celltype_final)
table1$antibody <- ""
for (i in 1:length(barcodes)){
	table1$antibody[i] <- paste(antibodies[which(CellRangerADT[,i] > opt$threshold, arr.ind=T)],collapse=",")
}
write.table(table1, file=paste(outprefix,"antibodies_and_celltypes_per_cell.tsv",sep="."), sep="\t", quote=FALSE)

###
# Create a table containing fraction of cells associated with a Celltype expressing an antibody.
print("Create a table containing fraction of cells associated with a Celltype expressing an antibody.")

table2 <- as.data.frame(table(table1$celltype_final))
print("Test2")
colnames(table2) <- c("Celltype","NumberOfCells")
table2 <- table2[which(table2$NumberOfCells > 0),]
#Initiate empty antibody columns:
for (ab in antibodies){
  table2[,ab] <- 0
}
# Loop through celltypes & Antibodies
for (ab in antibodies){
  for (ct in table2$Celltype){
    print(ct)
    occurence <- 0
    for (item in table1[which(table1$celltype_final==ct),]$antibody){
	if (grepl(paste(ab,",",sep=""), item) |grepl(paste(ab,"$",sep=""), item) ){
        occurence <- occurence +1
      }
    }
    #print(length(table1[which(table1$celltype_final==ct),]$antibody))
    fraction=occurence/length(table1[which(table1$celltype_final==ct),]$antibody)
    table2[which(table2$Celltype==ct), ab] <- round(fraction,3)
  }
}
write.table(table2, file=paste(outprefix,"antibody_fraction_per_celltype.tsv",sep="."), sep="\t", quote=FALSE, row.names = FALSE)


###
# Plot Antibodies per Celltype
# Since this are potentially many plots, create an extra subfolder for the output.
dir.create(paste(outfolder, "/antibodies_per_celltype_plots/", sep=""))
plot_data <- table2
plot_data$NumberOfCells <- NULL
plot_data <- melt(plot_data)

plot_antibody<-list()

for (celltype in unique(plot_data$Celltype)){ 
    print(celltype)
  a <- ggplot() + geom_bar(aes(x=variable, y=value), color="black", fill=ct.color[celltype], data=plot_data[which(plot_data$Celltype==celltype),], stat="identity") + ggtitle(celltype) + theme(
    plot.title = element_text(hjust = 0.5, size = 10),
    plot.margin = margin(1, 1, 1, 1, "pt"),
    #legend.text = element_text(size = 8),
    legend.key.size = unit(1, "line"),
    legend.text = element_text(size = rel(0.7)),
    legend.margin = margin(0.2, 0.2, 0.2, 0.2),
    axis.text = element_text(size = rel(0.4), angle=45, hjust=1),
    axis.title.x=element_blank(),
    axis.title.y=element_blank()
  )
  ggsave(
  paste(outfolder,"/antibodies_per_celltype_plots/", celltype,".png",sep=""), a,
  dpi = 600, width = 24, height = 14,units = "cm"
)
}
###
# Plot Antibodies per Celltype
# Since this are potentially many plots, create an extra subfolder for the output.
dir.create(paste(outfolder, "/antibodies_per_celltype_plots_neu/", sep=""),showWarnings=FALSE)
df_colData_cells <- as.data.frame(colData(my_sce), stringsAsFactors = FALSE)
CellRangerADTextended <- rownames_to_column(as.data.frame(t(CellRangerADT)))
colnames(CellRangerADTextended)[1] <- "barcodes"
cell_attributes <- plyr::join(df_colData_cells,CellRangerADTextended, by = "barcodes")

plot_antibody<-list()

id.final.ct = match(levels(cell_attributes$celltype_final), names(ct.color))

for (ab in antibodies){
    plot_data <- select(cell_attributes,celltype_final)
    plot_data$antibody <- cell_attributes[,ab]
    colnames(plot_data) <- c("celltype_final","antibody")
    plot_data$antibody[plot_data$antibody==0] <- NA
    plot_data$ln_umi <- log(plot_data$antibody) 
    plot_data<-plot_data[complete.cases(plot_data),]
    s <-ggplot(plot_data, aes(x = ln_umi, y = celltype_final, fill = celltype_final, height=..ndensity..))+ 
          geom_density_ridges(alpha=0.82,jittered_points = TRUE, point_alpha=0.4,  point_size = 0.4) +
          theme_ridges() +
         scale_x_continuous(limits = c(0,max(plot_data$ln_umi)+1)) +
          ggtitle(ab)+
         guides(fill=guide_legend(ncol=1))+
          scale_fill_manual(name="Cell type", values = ct.color[id.final.ct],drop=F)+ 
          theme(axis.title.y = element_blank(),plot.margin=unit(c(0.5,0.5,0.5,1),"cm"))
    ggsave(paste(outfolder,"/antibodies_per_celltype_plots_neu/", ab,".ridgeplots.png",sep=""), s,dpi = 600, width = 24, height = 28,units = "cm")
}


###
# Prepare data for the main plot showing gex vs. ab expression and the third table 

# get UMAP coordinates into data frame
umap_coord <- as.data.frame(reducedDims(my_sce)$umap_hvg)
names(umap_coord) <- c("umap1", "umap2")
umap_coord$barcodes <- rownames(umap_coord)

# make sure all cell dimensions are the same
stopifnot(my_sce$barcodes == colData(my_sce)$barcodes)
stopifnot(my_sce$barcodes == umap_coord$barcodes)
# add umap coordinates to the meta data table of cells
cell_attributes <- plyr::join(as.data.frame(colData(my_sce)), umap_coord)
print("test3")
# extend cell_attributes with ab info
print(lookup$Antibody)
antibodies <- unique(lookup$Antibody)
print(antibodies)
dfpanel <- data.frame(matrix(ncol=length(antibodies), nrow=length(barcodes)))
colnames(dfpanel) <- antibodies
rownames(dfpanel) <- barcodes
for (ab in antibodies){
  print(ab)
  for (i in 1:length(barcodes)){
    if (CellRangerADT[ab,barcodes[i]]>opt$threshold) {
      dfpanel[barcodes[i],ab]<- CellRangerADT[ab,barcodes[i]]
    } else {dfpanel[barcodes[i],ab]<- 0}
  }}
 dfpanel$barcodes <- rownames(dfpanel)
cell_attributes_extended <- join(cell_attributes,dfpanel)
cell_attributes_extended <- cell_attributes_extended[complete.cases(cell_attributes_extended),]


###
# Table 3
count_matrix = assay(my_sce, "normcounts")
table3<- lookup
table3$number_cells_per_AB <- 0
table3$fraction_total_cells_with_ab <- 0
table3$genes_expressed <- 0
table3$genes_not_expressed <- 0
table3$fraction_ab_labeled_cells_expressing_gene <- 0
colnames(table3)<- c("Gene", "Antibody","number_cells_per_AB","fraction_total_cells_with_ab","genes_expressed","genes_not_expressed", "fraction_ab_labeled_cells_expressing_gene")
print("mabye there is a bug here")
for (ab in seq(length(lookup$Antibody))) {
  count_expressed = 0
  count_not_expressed = 0
  count_ab = 0
  print(ab)
  for (index in seq(length(cell_attributes_extended$barcodes))) {
    total<-length(which(cell_attributes_extended[, lookup[ab,]$Antibody]>opt$threshold))
    if (cell_attributes_extended[index, lookup[ab,]$Antibody] > opt$threshold) {
      count_ab <- count_ab+1
      barcode <- cell_attributes_extended$barcodes[index]
      gene <- lookup[ab, ]$Gene
      if (is.na(match(gene, rownames(count_matrix)))) {
        print(paste(gene, " can not be found in the sce object.", sep=" "))
	next
      }
      else if (count_matrix[gene, barcode] != 0) {
        count_expressed = count_expressed + 1
      }
      else if (count_matrix[gene, barcode] ==0) {
        count_not_expressed = count_not_expressed + 1
      }
    }
  }
  count_not_expressed <- count_ab - count_expressed
  table3[ab,]$genes_not_expressed <- count_not_expressed
  table3[ab,]$genes_expressed <- count_expressed
  table3[ab,]$fraction_ab_labeled_cells_expressing_gene <- round(count_expressed/(count_ab),2)
  table3[ab,]$number_cells_per_AB <- count_ab
  table3[ab,]$fraction_total_cells_with_ab <- round(count_ab/length(cell_attributes_extended$barcodes),2)
}
print("Was there a bug?")
write.table(table3, file=paste(outprefix,"fraction_of_genes_expressed_per_antibody.tsv",sep="."), sep="\t", row.names=FALSE, quote=FALSE)


###
# Plot
# The following part has been copied to a large extend from the scPipeline. 
# Since we don't deal with multiple groups here, the code could be much simplified.
# TODO!!!
print("Antibody Gene Expression - Plots are created.")

group<-rep(c("has_corresponding_antibody"),length(unique(lookup$Gene)))
group_name <- "has_corresponding_antibody"
gene.list.all <- unique(lookup$Gene)
gene.list.all <- data.frame(Gene=unique(lookup$Gene), group)

id.first.ct = match(levels(cell_attributes$celltype_major), names(ct.color))
id.first.ct
id.final.ct = match(levels(cell_attributes$celltype_final), names(ct.color))
id.final.ct
# have matrix with normcounts
# remove all genes that have 0 counts in all cells
normcounts_all.zero.removed <- normcounts(my_sce)
mask_all_zero <- apply(normcounts_all.zero.removed, 1, sum) > 0
normcounts_all.zero.removed <- normcounts_all.zero.removed[mask_all_zero,]
stopifnot(length(normcounts_all.zero.removed[,1]) == sum(apply(normcounts_all.zero.removed, 1, sum) > 0))

# plot expression of antibody in UMAP

# keep only genes that are in matrix
gene.list = gene.list.all[gene.list.all$Gene %in% rownames(normcounts_all.zero.removed), ]
# have list per group
gene.list = tapply(gene.list$Gene, gene.list$group, c)
gene.list.all = tapply(gene.list.all$Gene, gene.list.all$group, c)
# sort gene names alphabetically for better overview in plot
gene.list = lapply(gene.list, sort)
gene.list.all = lapply(gene.list.all, sort)


# get indices of genes in matrix
list_groups <- lapply(seq_along(gene.list), function(x) {
  indices <-
    match(gene.list[[x]], rownames(normcounts_all.zero.removed))
  indices
})
names(list_groups) <- names(gene.list)

# sort by position in matrix, do sanity check with old list
list_groups_sorted <- lapply(list_groups, sort)
list_groups_orig = ids2indices(gene.list,
                               rownames(normcounts_all.zero.removed),
                               remove.empty = F)
print("str(list_groups_orig):")
print(str(list_groups_orig))
stopifnot(all.equal(list_groups_sorted, list_groups_orig))

# loop over groups of marker genes to plot gene expression on tSNEs with one png file per group

fontsize = theme(axis.text = element_text(size = 3), axis.title = element_text(size =3))
theme_set(theme_bw(4) + fontsize)

# plot UMAP (based on highly variable genes), colours = final cell type, as reference for expression plots
p_final_ref = ggplot(cell_attributes, aes(x=umap1, y=umap2, color=celltype_final)) +
  scale_color_manual(name="Cell type", values = ct.color[id.final.ct], drop=F)+
  geom_point(size = 0.0001) +
  theme(aspect.ratio=1)+
  #coord_fixed(ratio = 1) +
  xlab("UMAP 1") + ylab("UMAP 2") + theme(legend.position="none")
p_final_ref
p_legend_ref = ggplot(cell_attributes, aes(x=umap1, y=umap2, color=celltype_final)) +
  scale_color_manual(name="Cell type", values = ct.color[id.final.ct], drop=F)+
  geom_point(size = 1) +
  xlab("UMAP 1") + ylab("UMAP 2") +
  guides(colour = guide_legend(override.aes = list(size=0.8, shape=15), nrow = 17)) +
  theme(legend.key.width = unit(0.05,"line" ),
        legend.key.height = unit(0.001, "line"),#,
        #legend.text = element_text(size = 3.5),
        legend.spacing.y = unit(0.001, "line"),
        legend.spacing.x = unit(0.25, "line"))
p_legend_ref
legend_ref <- cowplot::get_legend(p_legend_ref)


# plot UMAP (based on highly variable genes), colours = final cell type, as reference for expression plot
# get all genes that have zero counts in all cells
mask.not.expressed <- !(gene.list.all[[group_name]] %in% rownames(normcounts_all.zero.removed[list_groups[[group_name]], , drop = F]))
genes.not.expressed <- sort(gene.list.all[[group_name]][mask.not.expressed])
genes.not.expressed.split <- split(genes.not.expressed, ceiling(seq_along(genes.not.expressed) / 7))
genes.not.expressed.split.collapsed <- lapply(seq_along(genes.not.expressed.split), function(x) {
    paste(genes.not.expressed.split[[x]], collapse = ", ")
  })

for (gene in genes.not.expressed){
	print(gene)
}

# generate plot that consists of text and lists all genes that were not expressed in this sample

if (length(genes.not.expressed.split.collapsed) > 0) { 
  text1 <-paste(genes.not.expressed.split.collapsed, collapse = ",\n")
} else { 
  text1 <- "-"}
text2 <- paste0("For the following genes \n no expression was detected in this sample:\n", text1)

plot_missed <- ggplot() +
  annotate( "text", x = 1, y = 1, label = text2, size = 1) +
  theme(
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    axis.title = element_blank(),
    axis.text = element_blank(),
    axis.ticks = element_blank(),
    aspect.ratio = 1
  )
plot_missed
# check if there are genes expressed and a submatrix can be generated

if (length(list_groups[[group_name]]) > 0) {
  # get matrix with all cells and all genes of the respective group
  matrix_group = normcounts_all.zero.removed[list_groups[[group_name]], , drop =
                                               F]
} else{
  matrix_group = matrix(0, nrow = 1, ncol = ncol(normcounts(my_sce)))
}

# check first if any of the group's genes are expressed in this sample
lookup_expressed<- lookup[-which(lookup$Gene %in% genes.not.expressed), ]
lookup_not_expressed<- lookup[which(lookup$Gene %in% genes.not.expressed), ]
rownames(lookup_expressed)<-c(seq(1:length(lookup_expressed$Gene)))
rownames(lookup_not_expressed)<-c(seq(1:length(lookup_not_expressed$Gene)))

# get empty list that will be filled with expression plots per gene
plot_gene = list()
lookup <- lookup[with(lookup, order(Gene)),]
lookup_expressed <- lookup_expressed[with(lookup_expressed, order(Gene)),]
lookup_not_expressed <- lookup_not_expressed[with(lookup_not_expressed, order(Gene)),]

print(lookup)
if (length(list_groups[[group_name]]) > 0) {
  # loop over the expressed genes of the current group
  for (gene in seq(length(lookup_expressed$Gene))) {
    # empty list that will be filled with 1) expression plot and 2) violin plot of one gene
    merged_plots <- list()
    gene_name=lookup_expressed$Gene[gene]
    # combine cell_attributes with expression of current gene
    cell_attributes_gene = cell_attributes
    cell_attributes_gene$normcounts = matrix_group[gene_name, ]
    # maximum count found for current gene
    max_count <- max(cell_attributes_gene$normcounts)
    # upper limit of gene expression colour scale is either maximum count or 3
    upper_limit <- max(3, max_count)
    legend = gene_name
    # plot gene expression
    merged_plots[["expr"]] = ggplot(cell_attributes_gene, aes(x = umap1, y = umap2)) +
       geom_point(data=cell_attributes_gene[which(cell_attributes_gene$normcounts==0),], size = rel(0.001), colour="slateblue4")+
      geom_point(data=cell_attributes_gene[which(cell_attributes_gene$normcounts>0),],aes(color = normcounts), size = rel(0.001)) +
      xlab("UMAP 1") + ylab("UMAP 2") +
#      scale_color_gradient( name = "", low = "grey90",high = "seagreen",limits = c(0, max(3, upper_limit)), breaks = c(floor(upper_limit / 3), round(2 *(upper_limit / 3)), upper_limit)) +
      scale_color_gradientn(name="", colours = c("slateblue3", "royalblue1", "aquamarine3","khaki", 383, "sienna1", "orangered4"),limits = c(1, max(3, upper_limit)), breaks = c(floor(upper_limit / 3), round(2 *(upper_limit / 3)), upper_limit))+
      coord_fixed(ratio = 1) +
      ggtitle(legend) +
      theme(
        plot.title = element_text(
          face = "bold",
          hjust = 0.5,
          size = 4
        ),
        plot.margin = margin(1, 1, 1, 1, "pt"),
        plot.background = element_rect(
          fill = "white",
          colour = "black",
          size = 0.2
        ),
        #legend.text = element_text(size = 8),
        legend.key.size = unit(0.3, "line"),
        legend.text = element_text(size = rel(0.7)),
        legend.margin = margin(0.2, 0.2, 0.2, 0.2),
        axis.text = element_text(size = rel(0.6)),
        aspect.ratio = 1
      )
    merged_plots[["expr"]]
    #scale_color_distiller(palette="RdBu")
    # combine cell_attributes with antibody
    cell_attributes_antibody = cell_attributes_extended
    cell_attributes_antibody$expressed = as.numeric(cell_attributes_extended[, lookup_expressed[gene, ]$Antibody])
    #cell_attributes_antibody$expressed[cell_attributes_antibody$expressed==0] <- NA
    cell_attributes_antibody$expressed
    # plot antibody plot
    upper_limit=round(unname(quantile(cell_attributes_antibody$expressed,c(.99),na.rm=TRUE)))
    upper_break=max(opt$threshold*3, upper_limit)
    medium_break=round(2 *( max(opt$threshold*3, upper_limit) / 3))
    lower_break=floor( max(opt$threshold*3, upper_limit)/ 3)
   merged_plots[["antibody"]]= ggplot(cell_attributes_antibody,
       aes(x = umap1, y = umap2, color = expressed)) +
      geom_point(data=cell_attributes_antibody[which(cell_attributes_antibody$expressed==0),], size = rel(0.001), colour="gray66") +
      geom_point(data=cell_attributes_antibody[which(cell_attributes_antibody$expressed>0),], na.rm=TRUE,size = rel(0.001)) + xlab("UMAP 1") + ylab("UMAP 2") +
      geom_point(data=cell_attributes_antibody[which(cell_attributes_antibody$expressed>max(opt$threshold*3, upper_limit)),],size = rel(0.001), colour="orangered4")+
#	scale_color_viridis(option = "plasma", name="",limits = c(5, max(3, upper_limit)), breaks = c(floor(upper_limit / 3), round(2 *(upper_limit / 3)), upper_limit))+
  scale_color_gradientn(name="",  colours = c("slateblue3", "royalblue1", "aquamarine3","khaki", 383, "sienna1", "orangered4"),limits = c(opt$threshold, max(opt$threshold*3, upper_limit)), breaks = c(lower_break, medium_break, upper_break), labels=c(lower_break, medium_break, paste(">=" ,upper_break,sep="")))+
        ggtitle(lookup_expressed[gene, ]$Antibody) +
        coord_fixed(ratio = 1) +
        theme(
        plot.title = element_text(
          face = "bold",
          hjust = 0.5,
          size = 4
        ),
        plot.margin = margin(1, 1, 1, 1, "pt"),
        plot.background = element_rect(
          fill = "white",
          colour = "black",
          size = 0.2
        ),
        #legend.text = element_text(size = 8),
        legend.key.size = unit(0.3, "line"),
        legend.text = element_text(size = rel(0.7)),
        legend.margin = margin(0.2, 0.2, 0.2, 0.2),
        axis.text = element_text(size = rel(0.6)),
        aspect.ratio = 1,
        axis.title = element_text(size = 3.5)
      )
    merged_plots[["empty"]]<-ggplot() + theme_void()
    # combine expression plot and antibody plot of current gene to one plot
    plot_gene[[gene]] = plot_grid(
      plotlist = merged_plots,
      ncol = 1,
      align = "v",
      rel_heights = c(2.1, 2.1,0.8)
   )
  }
}

#ggplot2::ggsave(filename= paste(outprefix,"ridgeplot.png",sep="."),plot = rigdePlot)
dir.create(paste(outfolder, "/ExpressionPlots/", sep=""))
expout=paste(outfolder, "/ExpressionPlots/",opt$sampleName,sep="")

for (i in seq(1,20*ceiling(length(plot_gene)/20),20)){
	plot_gene_sub = c(list(plot_missed, p_final_ref, legend_ref), plot_gene[i:min(i+20,length(plot_gene))])
	if (length(plot_gene_sub) < 15){
		plot_group =  plot_grid(plotlist = plot_gene_sub, ncol = 4)}
	else {
	plot_group =  plot_grid(plotlist = plot_gene_sub, ncol = 6) }
	ggsave(paste(expout,"plot",i,"to_plot",i+20,"gene_antibody_expression.png",sep="."),
            plot_group,
            dpi = 600,
            width = 30,
            height = 30,
            units = "cm"
)
}


#names(plot_gene) = rownames(matrix_group)
#plot_gene = c(list(plot_missed, p_final_ref, legend_ref), plot_gene)
#nr.cols = min(8, 2 * (length(list_groups[[group_name]]) + 3))
#nr.rows = ceiling((length(list_groups[[group_name]]) + 3) / nr.cols)
#plots_group = plot_grid(plotlist = plot_gene, ncol = nr.cols)
# save png for each group
#ggsave(
#  paste(outprefix,"gene_antibody_expression.png",sep="."),
#  plots_group,
#  dpi = 600,
#  width = 25,
#  height = 2 * nr.rows,
#  units = "cm"
#)


### 
# Plot those for which no gene expression has been found. 

# get empty list that will be filled with expression plots per gene
plot_gene_missed = list()

if (length(genes.not.expressed) > 0) {
  # loop over the expressed genes of the current group
  for (gene in seq(length(lookup_not_expressed$Gene))) {
    # empty list that will be filled with 1) expression plot and 2) violin plot of one gene
    missed_plots <- list()
    gene_name=lookup_not_expressed$Gene[gene]
    # combine cell_attributes with expression of current gene
    # combine cell_attributes with antibody
    cell_attributes_antibody = cell_attributes_extended
    cell_attributes_antibody$expressed = as.numeric(cell_attributes_extended[, lookup_not_expressed[gene, ]$Antibody])
    cell_attributes_antibody$expressed[cell_attributes_antibody$expressed==0] <- NA
    # plot antibody plot
   # plot antibody plot
upper_limit=round(unname(quantile(cell_attributes_antibody$expressed,c(.99),na.rm=TRUE)))
    upper_break=max(opt$threshold*3, upper_limit)
    medium_break=round(2 *( max(opt$threshold*3, upper_limit) / 3))
    lower_break=floor( max(opt$threshold*3, upper_limit)/ 3)
    missed_plot= ggplot(cell_attributes_antibody,
       aes(x = umap1, y = umap2, color = expressed)) +
      geom_point(alpha = 0.7, size = rel(0.001)) + xlab("UMAP 1") + ylab("UMAP 2") +
	 scale_color_gradientn(name="", na.value="#DCDCDC00",colours = c("slateblue4", "royalblue1", "aquamarine3","khaki", 383, "sienna1", "orangered4"),limits = c(0, max(3, upper_limit)), breaks = c(floor(upper_limit / 3), round(2 *(upper_limit / 3)), upper_limit))+
        ggtitle(lookup_not_expressed[gene, ]$Antibody) +
        coord_fixed(ratio = 1) +
        theme(
        plot.title = element_text(
          face = "bold",
          hjust = 0.5,
          size = 4
        ),
        plot.margin = margin(1, 1, 1, 1, "pt"),
        plot.background = element_rect(
          fill = "white",
          colour = "black",
          size = 0.2
        ),
        #legend.text = element_text(size = 8),
        legend.key.size = unit(0.3, "line"),
        legend.text = element_text(size = rel(0.7)),
        legend.margin = margin(0.2, 0.2, 0.2, 0.2),
        axis.text = element_text(size = rel(0.6)),
        aspect.ratio = 1,
        axis.title = element_text(size = 3.5)
      )
    merged_plots <- list()
    merged_plots[["antibody"]]<-missed_plot
    merged_plots[["empty"]]<-ggplot() + theme_void()
    # combine expression plot and antibody plot of current gene to one plot
    plot_gene_missed[[gene]] = plot_grid(
      plotlist = merged_plots,
      ncol = 1,
      rel_heights = c(2.1, 0.3)
   )
  }
}
names(plot_gene_missed) = rownames(genes.not.expressed)
plot_gene_missed = c(list(plot_missed, p_final_ref, legend_ref), plot_gene_missed)
plots_missed = plot_grid(plotlist = plot_gene_missed, ncol = 6)
# save png for each group
ggsave(
  paste(expout,"antibody_no_gene_expression.png",sep="."),
  plots_missed,
  dpi = 600,
  width = 25,
  height = 30,
  units = "cm"
)



