##Script to perform the full differential expression analysis on ADT and GEX data

## Example Call
#Rscript differential_expression.R --seurat_object PBMC_healthy.integrated_seurat_object.RDS

library(Seurat)
library(optparse)

option_list = list(
make_option("--seurat_object", type = "character", default = "PBMC_healthy.integrated_seurat_object.RDS"))
opt_parser = OptionParser(option_list = option_list)
opt = parse_args(opt_parser)

seurat_integrated=opt$seurat_object
seurat_integrated=readRDS("PBMC_healthy.integrated_seurat_object.RDS")

tmp=as.character(seurat_integrated$sample_name)
tmp=gsub("_diss","D",tmp)
tmp=unlist(lapply(strsplit(tmp, "_"),function(x)x[3]))

seurat_integrated$sample_type=tmp


Idents(seurat_integrated)=seurat_integrated$sample_type


tmp=unique(Idents(seurat_integrated))


markers_gex_pairwise=list()
markers_gex_all=list()
markers_adt_pairwise=list()
markers_adt_all=list()

#loop by celltype and do all sample types pairwise
#Then do also each sample type against all the others

bbb=1
ccc=1
aaa=unique(seurat_integrated$celltype_major)

bbb=0
ccc=0
for(k in 1:length(aaa)){
  markers_gex_pairwise=NULL
  markers_adt_pairwise=NULL
  markers_gex_all=NULL
  markers_adt_all=NULL
  message(paste("Working on celltype:", aaa[k]))

  seurat_subset=subset(seurat_integrated,subset=(celltype_major==aaa[k]))

  message("contrast skinD vs Untreated")
  markers_gex_pairwise=FindMarkers(object=seurat_subset, ident.1="skinD", ident.2="untreated",  assay="RNA", logfc.threshold=0, min.cells.group=1,min.cells.feature=1,min.pct=0,only.pos=FALSE)
  markers_adt_pairwise=FindMarkers(object=seurat_subset, ident.1="skinD", ident.2="untreated", assay="adt", logfc.threshold=0, min.cells.group=1,min.cells.feature=1,min.pct=0,only.pos=FALSE)
  tmpnamegex=paste(aaa[k], "- skinD vs untreated gex")
  tmpnameadt=paste(aaa[k], "- skinD vs untreated adt")
  saveRDS(markers_gex_pairwise, tmpnamegex)
  saveRDS(markers_adt_pairwise, tmpnameadt)

  message("contrast tumorD vs Untreated")
  markers_gex_pairwise=FindMarkers(object=seurat_subset, ident.1="tumorD", ident.2="untreated",  assay="RNA", logfc.threshold=0, min.cells.group=1,min.cells.feature=1,min.pct=0,only.pos=FALSE)
  markers_adt_pairwise=FindMarkers(object=seurat_subset, ident.1="tumorD", ident.2="untreated", assay="adt", logfc.threshold=0, min.cells.group=1,min.cells.feature=1,min.pct=0,only.pos=FALSE)
  tmpnamegex=paste(aaa[k], "- tumorD vs untreated gex")
  tmpnameadt=paste(aaa[k], "- tumorD vs untreated adt")
  saveRDS(markers_gex_pairwise, tmpnamegex)
  saveRDS(markers_adt_pairwise, tmpnameadt)
		
  message("contrast skinD vs tumorD")
  markers_gex_pairwise=FindMarkers(object=seurat_subset, ident.1="skinD", ident.2="tumorD",  assay="RNA", logfc.threshold=0, min.cells.group=1,min.cells.feature=1,min.pct=0,only.pos=FALSE)
  markers_adt_pairwise=FindMarkers(object=seurat_subset, ident.1="skinD", ident.2="tumorD", assay="adt", logfc.threshold=0, min.cells.group=1,min.cells.feature=1,min.pct=0,only.pos=FALSE)
  tmpnamegex=paste(aaa[k], "- skinD vs tumorD gex")
  tmpnameadt=paste(aaa[k], "- skinD vs tumorD adt")
  saveRDS(markers_gex_pairwise, tmpnamegex)
  saveRDS(markers_adt_pairwise, tmpnameadt)
	
}

