##Script to create dotplots base don a provided list of epitopes and the expression saved in the provided integrated Seurat object

 library(Seurat)
  library(plyr)
  library(dplyr)
  library(SingleCellExperiment)
  library(optparse)
  library(reticulate)
  library(ggplot2)
  library(reshape2)
  library(cowplot)
  library(Matrix)
  library(patchwork)
  library(tidyverse)


thresholds <- read.csv("/cluster/project/nexus/phrt/levesque_melanoma_2019/git/metadata/thresholds.solid.extended_panel.txt", sep= "\t", stringsAsFactors = FALSE)
thresholds$Antibody<- sub("_", "-", thresholds$Antibody)
rownames(thresholds) <- toupper(thresholds$Antibody)
print(tail(thresholds,20))

DefaultAssay(seurat_integrated) <- "adt"

my_adt <- t(seurat_integrated@assays$adt@counts)
meta <- as.data.frame(seurat_integrated@meta.data)
my_merge <- merge(my_adt,meta,by=0)
my_merge_to_compare <- my_merge 

head(my_merge)

unique(my_merge$sampleID)
for (ab in thresholds$Antibody){
  print(ab)
  for (sample  in unique(my_merge$sampleID)){
	  if(is.null(my_merge[my_merge$sampleID==sample,paste0("ADT-",ab)])){
		  message(paste("ab", ab, "not in the table. Skipping"))
		  next()
	  }
    my_merge[my_merge$sampleID==sample,paste0("ADT-",ab)] <- my_merge[my_merge$sampleID==sample,paste0("ADT-",ab)] - exp(thresholds[toupper(ab), sample])
    set_to_0 <- my_merge[my_merge$sampleID==sample,paste0("ADT-",ab)] <0
    my_merge[my_merge$sampleID==sample,paste0("ADT-",ab)][set_to_0] <-0
  }
  
}
DefaultAssay(seurat_integrated) <- "adt_thresholds"
#Missing the hastags, not a problem for this specific plot

seurat_integrated_bck=seurat_integrated
# TODO
# Adapt this so it works for different panels & read in marker gene / protein list.
my_new_assay <- my_merge[,1:123]
dim(my_new_assay)
rownames(my_new_assay) <- my_new_assay$Row.names
my_new_assay$Row.names <- NULL

seurat_integrated@assays$adt_thresholds <-  CreateAssayObject(counts = t(my_new_assay))
Idents(seurat_integrated) <-seurat_integrated$celltype_final

tokeep=rep("", length(seurat_integrated$sample_name))
tokeep[which(seurat_integrated$sample_name%in%c("HS3_1","HS3_2","HS3_4"))]<-"x"
seurat_integrated<-AddMetaData(seurat_integrated, metadata=tokeep, col.name="tokeep")
 tmp=SplitObject(seurat_integrated, split.by="tokeep")
seurat_integrated<-tmp$x


tmp <- as.character(seurat_integrated$celltype_major)
 tmp[grep("Fibroblasts",tmp) ]<-"Fibroblasts"
tmp[grep("Keratinocytes",tmp) ]<-"Keratinocytes"
tmp[grep("Macrophages",tmp) ]<-"Macrophages"
tmp[grep("Melanoma",tmp) ]<-"Melanoma"
tmp[grep("Pericyte",tmp) ]<-"Pericyte"
seurat_integrated$collapse_celltype <- tmp

seurat_subset=subset(seurat_integrated, subset=celltype_major != "unknown")
seurat_subset=subset(seurat_subset, subset=celltype_major != "uncertain")


epitopes <- c("HLA-A-B-C","HLA-DR","CD45","CD3","CD103","CD1c","CD31","Podoplanin","CD326","CD146","CD117","CD10","CD90")
epitopes <- paste0("ADT-",epitopes)

Idents(seurat_subset)<-seurat_subset$collapse_celltype
idents <- sort(Idents(seurat_subset))


DotPlot_adtThresholds <- DotPlot(seurat_subset, features = epitopes,
                                 idents = idents,#c("B.cells","Dendritic.cells", "Dendritic.cells.Plasmacytoid","T.cells","T.cells.CD8","T.cells.CD4.naive", "NK.cells", "Monocytes"), 
                   assay="adt_thresholds", dot.scale = 10,scale = FALSE, scale.by = "size") + RotatedAxis()  + scale_colour_gradient2(low="gray", mid="lightblue", high="darkblue",na.value="darkblue")
ggsave(filename = paste0(opt$outdir, opt$outName, "_DotPlot_ADT_fig6D_PBMC_v2.png"),
       width = 40, height = 20, dpi = 600, units = "cm", plot=DotPlot_adtThresholds)
#

epitopes2 <- c("HLA-A","HLA-DRA","PTPRC","CD3E","ITGAE","CD1C","PECAM1","PDPN","EPCAM","MCAM","KIT","MME","THY1")
DotPlot_gex <- DotPlot(seurat_subset, features = epitopes2,#c("PTPRC","MS4A1","CD19","CD22","NT5E","CD1C","THBD","IL3RA","CD14","CD33","CD86","ITGAM","KLRF1","FCGR3A","B3GAT1","CD3E","CD4","CD8A","FOXP3","IL2RA"), 
                                 idents = idents,#c("B.cells","Dendritic.cells", "Dendritic.cells.Plasmacytoid","T.cells","T.cells.CD8","T.cells.CD4.naive", "NK.cells"), 
                                 assay="SCT", dot.scale = 10, col.min = 0,scale.by = "size") + RotatedAxis()  + scale_colour_gradient2(low="gray", mid="lightblue", high="darkblue",na.value="darkblue") 
#DotPlot_gex
ggsave(filename = paste0(opt$outdir, opt$outName, "_DotPlot_GEX_fig6E.png"),
       width = 40, height = 20, dpi = 600, units = "cm", plot = DotPlot_gex)



