## Script to plot combined ridgeplot over multiple used tags in one project.

## To do: extend script that it can be used on multiple projecect having multiple tags.

# Linda Grob, 01.2020
# Example call
#Rscript plotRidgeplots.PBMCmix.20191130.R --atypicalRemoved /cluster/project/nexus/phrt/levesque_melanoma_2019/data/20191130/PBMC_mix/analysis_PBMC001/atypical_removed/,/cluster/project/nexus/phrt/levesque_melanoma_2019/data/20191130/PBMC_mix/analysis_PBMC002/atypical_removed/,/cluster/project/nexus/phrt/levesque_melanoma_2019/data/20191130/PBMC_mix/analysis_PBMC003/atypical_removed/ --analysisADT /cluster/project/nexus/phrt/levesque_melanoma_2019/data/20191130/PBMC_mix/analysis_pooledSample/CellRangerADT/PBMCmix_pooled/outs/filtered_feature_bc_matrix/ --outfolder /cluster/project/nexus/phrt/levesque_melanoma_2019/data/20191130/PBMC_mix/CompareRidgeplots/ --sampleNames PBMC001,PBMC002,PBMC003



library(cowplot)
library(ggridges)
library(optparse)
library(Seurat)
library(rjson)
library(plyr)
library(dplyr)
library(reshape)
library(ggplot2)
library(limma)
library(viridis)
library(grDevices)

option_list = list(
make_option("--atypicalRemoved", type = "character", help = "Comma separated list with Paths to the atypicalRemoved Folder/RDS Files of the GEX pipline."),
make_option("--analysisADT", type = "character", help = "Path to the pooled ADT analysis folder."),
make_option("--outfolder", type = "character", help = "Path to the outfolder."),
make_option("--sampleNames", type="character", help="SampleNames in the order of the atypicalRemoved Input Files.")

)
opt_parser = OptionParser(option_list = option_list)
opt = parse_args(opt_parser)

outfolder <- opt$outfolder
dir.create(outfolder, showWarnings=FALSE)
outprefix <- paste(outfolder, "CombinedRidgeplots", sep="")
# set general options/parameters
options(stringsAsFactors = FALSE)

# Read in files
#ADT
CellRangerADTfolder <- opt$analysisADT
CellRangerADT <- Read10X(data.dir = CellRangerADTfolder)
#GEX: Pre load all rds files, to avoid long runningtime.
my_folders <- as.list(strsplit(opt$atypicalRemoved,","))[[1]]
my_sampleNames <- as.list(strsplit(opt$sampleNames,","))[[1]]
my_sceList <- list()
for (tag_folder in 1:length(my_folders)){
    RDSfile <- list.files(my_folders[tag_folder], pattern="\\.RDS$")
    my_sceList[[tag_folder]] <- readRDS(paste(my_folders[tag_folder],RDSfile, sep=""))

}
#Extract Tag spezific ADT subset & Plot subplots
SubmatrixList <- list()

plot_list=list()
for (i in seq(1,length(rownames(CellRangerADT)))){
jointSet <- data.frame(barcode=character(),antibody=character(),value=numeric())
print(rownames(CellRangerADT)[i])
for (tag_folder in 1:length(my_folders)){
    my_sce <- my_sceList[[tag_folder]]
    joint.bcs <- intersect(colnames(my_sce), colnames(CellRangerADT))
    CellRangerADTsub <- CellRangerADT[,joint.bcs]
    ABsubset <- as.matrix(CellRangerADTsub[i,] )
    colnames(ABsubset) <- rownames(CellRangerADT)[i]
    ABsubset <- melt(ABsubset)
    colnames(ABsubset) <- c("barcode","antibody","value")
    ABsubset$sample <- my_sampleNames[[tag_folder]]
    ABsubset$value[ABsubset$value==0] <- NA
    ABsubset$log <- log(ABsubset$value)
    jointSet <- rbind(jointSet,ABsubset)
}
    s <-ggplot(jointSet, aes(x = log, y = sample, fill = sample,height=..ndensity..)) +
          geom_density_ridges(alpha=0.3) +
          scale_x_continuous(limits = c(0,max(jointSet$log)+1)) +
          #geom_vline(xintercept = log(threshold)) +
          theme_ridges() +
          ggtitle(rownames(CellRangerADT)[i]) +
          theme(legend.position = "none",axis.title.y = element_blank(),plot.margin=unit(c(0.5,0.5,0.5,1),"cm"))
    plot_list[[i]]<- s
}

# Save plots in combined plots on 4x4 grids.
numberAntibodies = length(rownames(CellRangerADT))
print(plot_list)
number=1
for (i in seq(1,15*ceiling(numberAntibodies/15),15)){	
        plot_sub <- plot_list[i:min(i+15,numberAntibodies)]
        plot_group =  plot_grid(plotlist = plot_sub, ncol = 4)
	ggplot2::ggsave(filename= paste(outprefix,"Part",number,"png",sep="."),width=21,height=14, plot = plot_group)
	number=number+1
}

