# This script loads thresholds lists and makes dedicated scatterplots to highlight the necessity of manual thresholding depending on the sample variables
# The script is not generalized as it's a custom downstream analysis catered to the specific samples and gene lists we want to highlight

library(reshape2)
library(ggplot2)

####
# PBMC only the untreated
pbmc <- read.delim("pbmc_thresholds.txt")
pbmc_to_use <- c("Antibody","PBMC_D1_untreated","PBMC_D2_untreated","PBMC_D3_untreated", "PBMC_IT_1", "PBMC_IT_2", "PBMC_IT_3")
pbmc <- pbmc[,which(colnames(pbmc)%in%pbmc_to_use)]
myvariances <- NULL

for(i in 1:nrow(pbmc)){
 tmp <- unlist(pbmc[i,2:ncol(pbmc)])
 myvariances <- c(myvariances, var(tmp))
}
names(myvariances) <- pbmc[,1]
myvariances <- sort(myvariances,decreasing=T)
top30 <- names(myvariances)[1:30]
top30 <- pbmc[which(pbmc[,1]%in%top30),]

pbmc_p <- melt(top30)
names(pbmc_p)[which(names(pbmc_p)=="value")]<-"threshold"
colnames(pbmc_p)[which(colnames(pbmc_p)=="variable")]="donor"
p <- ggplot(pbmc_p, aes(x=Antibody, y=threshold, color=donor)) + geom_jitter(position=position_jitter(0.2)) +
  theme(text = element_text(size = 15), axis.text.x = element_text(angle = 45, vjust = 1, hjust=1))
ggsave(plot=p, filename="pbmc_untreated_thresholds.pdf", dpi=600, width=50, height=10, units="cm")


# PBMC dissociations
pbmc <- read.delim("pbmc_thresholds.txt")
pbmc_to_use <- c("Antibody","PBMC_D1_untreated", "PBMC_D2_untreated", "PBMC_D3_untreated", "PBMC_D1_skin_diss","PBMC_D2_skin_diss","PBMC_D3_skin_diss","PBMC_D1_tumor_diss","PBMC_D2_tumor_diss","PBMC_D3_tumor_diss")
pbmc <- pbmc[,which(colnames(pbmc)%in%pbmc_to_use)]
myvariances <- NULL

for(i in 1:nrow(pbmc)){
 tmp <- unlist(pbmc[i,2:ncol(pbmc)])
 myvariances <- c(myvariances, var(tmp))
}
names(myvariances) <- pbmc[,1]
myvariances <- sort(myvariances,decreasing=T)
top30 <- names(myvariances)[1:30]
top30 <- pbmc[which(pbmc[,1]%in%top30),]

colnames(top30)[which(colnames(top30)=="PBMC_D1_skin_diss")]="PBMC_D1_skin_dissociation"
colnames(top30)[which(colnames(top30)=="PBMC_D2_skin_diss")]="PBMC_D2_skin_dissociation"
colnames(top30)[which(colnames(top30)=="PBMC_D3_skin_diss")]="PBMC_D3_skin_dissociation"
colnames(top30)[which(colnames(top30)=="PBMC_D1_tumor_diss")]="PBMC_D1_tumor_dissociation"
colnames(top30)[which(colnames(top30)=="PBMC_D2_tumor_diss")]="PBMC_D2_tumor_dissociation"
colnames(top30)[which(colnames(top30)=="PBMC_D3_tumor_diss")]="PBMC_D3_tumor_dissociation"

pbmc_p <- melt(top30)

names(pbmc_p)[which(names(pbmc_p)=="value")]<-"threshold"
colnames(pbmc_p)[which(colnames(pbmc_p)=="variable")]="donor"

p <- ggplot(pbmc_p, aes(x=Antibody, y=threshold, color=donor)) + geom_jitter(position=position_jitter(0.2)) +
  theme(text = element_text(size = 15), axis.text.x = element_text(angle = 45, vjust = 1, hjust=1))
ggsave(plot=p, filename="pbmc_dissociation_thresholds.pdf", dpi=600, width=50, height=10, units="cm")


#####
# One with all the solids, but HS1 and HS2
solid <- read.delim("solid_thresholds.txt")
solid_to_use <- c("Antibody","HS3_1","HS3_2","HS3_4", "LN_MM_1", "LN_MM2", "LN_MM_4", "M170929", "M180223","M181024")
solid <- solid[,which(colnames(solid)%in%solid_to_use)]
myvariances <- NULL

for(i in 1:nrow(solid)){
 tmp <- unlist(solid[i,2:ncol(solid)])
 myvariances <- c(myvariances, var(tmp))
}
names(myvariances) <- solid[,1]
myvariances <- sort(myvariances,decreasing=T)
top30 <- names(myvariances)[1:30]
top30 <- solid[which(solid[,1]%in%top30),]

colnames(top30)[which(colnames(top30)=="M170929")]="PM1"
colnames(top30)[which(colnames(top30)=="M180223")]="PM2"
colnames(top30)[which(colnames(top30)=="M181024")]="PM3"


solid_p<- melt(top30)
solid_p$variable = factor(solid_p$variable, levels=c("HS3_1","HS3_2","HS3_4", "LN_MM_1", "LN_MM2", "LN_MM_4","PM1","PM2", "PM3"))

names(solid_p)[which(names(solid_p)=="value")]<-"threshold"
colnames(solid_p)[which(colnames(solid_p)=="variable")]="donor"

p <- ggplot(solid_p, aes(x=Antibody, y=threshold, color=donor)) + geom_jitter(position=position_jitter(0.2)) +
  theme(text = element_text(size = 15), axis.text.x = element_text(angle = 45, vjust = 1, hjust=1))
ggsave(plot=p, filename="solid_all_thresholds.pdf", dpi=600, width=50, height=10, units="cm")



#HS using only HS1 and HS2
solid <- read.delim("solid_thresholds.txt")
solid_to_use <- c("Antibody","HS1_1","HS1_2","HS1_3", "HS2_1", "HS2_2", "HS2_4")
solid <- solid[,which(colnames(solid)%in%solid_to_use)]
myvariances <- NULL

for(i in 1:nrow(solid)){
 tmp <- unlist(solid[i,2:ncol(solid)])
 myvariances <- c(myvariances, var(tmp))
}
names(myvariances) <- solid[,1]
myvariances <- sort(myvariances,decreasing=T)
top30 <- names(myvariances)[1:30]
top30 <- solid[which(solid[,1]%in%top30),]

solid_p<- melt(top30)

tmp <- as.character(solid_p$variable)

hs=solid_p
tmp <- hs$variable

names(hs)[which(names(hs)=="value")]<-"threshold"
colnames(hs)[which(colnames(hs)=="variable")]="donor"
p <- ggplot(hs, aes(x=Antibody, y=threshold, color=donor)) + geom_jitter(position=position_jitter(0.2)) +
  theme(text = element_text(size = 15), axis.text.x = element_text(angle = 45, vjust = 1, hjust=1))

ggsave(plot=p, filename="healthy_skin_thresholds.pdf", dpi=600, width=50, height=10, units="cm")







