##Script to generate finalized custom liquid PBMC untreated UMAP

## Example Call
#Rscript plot_liquid_umaps.R --cohort_object PBMC_untreated.integrated_seurat_object.RDS --outName PBMC_untreated --outdir ./ --colour_config colour_config.merged.txt --add_meta metadata_table.txt --lookup reference_lookup.tab --thresholds thresholds.txt


 library(Seurat)
  library(plyr)
  library(dplyr)
  library(SingleCellExperiment)
  library(optparse)
  library(reticulate)
  library(ggplot2)
  library(reshape2)
  library(cowplot)
  library(Matrix)
  library(patchwork)
  library(tidyverse)
  library(optparse)

  # convenience function for string concatenation
"%&%" <- function(a, b) paste(a, b, sep = "")
empty_string <- ""

option_list = list(
make_option("--cohort_object", type = "character", default = "/cluster/project/nexus/phrt/levesque_melanoma_2019/cohort_integration/PBMC_untreated/analysis/PBMC_untreated.integrated_seurat_object.RDS"),
make_option("--outName", type = "character", default = "PBMC_untreated"),
make_option("--outdir", type = "character", default = "/cluster/project/nexus/phrt/levesque_melanoma_2019/cohort_integration/PBMC_untreated/analysis/"),
make_option("--colour_config", type = "character", default = "/cluster/project/nexus/phrt/levesque_melanoma_2019/cohort_integration/colour_config.merged.txt"),
make_option("--add_meta", type = "character", default = "metadata_table.txt"),
make_option("--lookup", type = "character", default = "/cluster/project/nexus/phrt/levesque_melanoma_2019/cohort_integration/reference_lookup.tab"),
make_option("--thresholds", type = "character", default = "thresholds.txt")
)
opt_parser = OptionParser(option_list = option_list)
opt = parse_args(opt_parser)



###################################
###   Set plotting Parameters   ###
###################################

# If one of the values plot_aspect_ratio, chunksize, or number of columns is changed in the following section
# the other values should be checked as well.
# Also, check physical size of the generated output file in the respective plotting section of the script.

# IMPORTANT: when generating UMAP plots with the function RunUMAP() from Seurat the parameters "spread", "min.dist" and "n.neighbors"
# have a great impact on the resulting plot. They should be checked and optimised for each project.

# for all UMAP plots the same aspect ratio is used to have a comparable visualisation
plot_aspect_ratio <- 0.7

### The selected genes and gene sets are plotted in chunks so that the file size is not too large:

# chunksize_genes determines the number of genes shown in one expression plot
chunksize_genes <- 9
# columns_genes determines the number of columns the gene expression plots are arranged in.
## E.g. "columns_genes <- 1" arranges all plots in the output file below one another in a single column.
columns_genes <- 3

# chunk size of gene sets in GSVA plots
chunksize_genesets <- 12
# number of columns in GSVA plots
columns_genesets <- 4

# override the theme change of cowplot
theme_set(theme_grey())
print(Sys.time())


########################
###   Read in data   ###
########################

# read in color config
#config <- read.csv("colour_config.merged.txt", sep = "\t", stringsAsFactors = FALSE)
config <- read.csv(opt$colour_config, sep = "\t", stringsAsFactors = FALSE)
print(config)

# read in seurat object
seurat_integrated <- readRDS(opt$cohort_object)
print(seurat_integrated)
cell_attributes <- as.data.frame(Embeddings(seurat_integrated, reduction = "wnn.umap"))
print(cell_attributes)
print(rownames(cell_attributes))
print(substring(rownames(cell_attributes),0,16))
print(unique(substring(rownames(cell_attributes),0,16)))
print(length(rownames(cell_attributes)))
print(length(unique(substring(rownames(cell_attributes),0,16))))
print(duplicated(substring(rownames(cell_attributes),0,16)))
cell_attributes <- cell_attributes[!duplicated(substring(rownames(cell_attributes),0,16)), ]
print(cell_attributes)
rownames(cell_attributes) <- substring(rownames(cell_attributes),0,16)
print(head(cell_attributes))


metadata <- seurat_integrated@meta.data
metadata <- metadata[!duplicated(substring(rownames(metadata),0,16)),]
print(head(rownames(metadata)))
rownames(metadata) <- substring(rownames(metadata),0,16)
print(head(rownames(metadata)))
print(colnames(metadata))
metadata$barcodes <- NULL
metadata$barcodes <- rownames(metadata)
print(colnames(metadata))
print(metadata$barcodes)
print(head(metadata))
cell_attributes$barcodes <- NULL
cell_attributes$barcodes <- rownames(cell_attributes)
cell_attributes <- plyr::join(cell_attributes, metadata, by="barcodes")
rownames(cell_attributes) <- cell_attributes$barcodes
cell_attributes$Row.names <- NULL
print(head(cell_attributes))



print(head(metadata))
number_samples <- length(unique(metadata$sampleID))
# optionally, read in an additional meta data table
print(empty_string)
 if (is.character(opt$add_meta)) {
   addMeta_table <- read.csv(opt$add_meta, sep = "\t", stringsAsFactors = FALSE)
   ## Use sampleID up to (but excluding) the second "." as shorter sampleID
   addMeta_table$sampleID <- regmatches(addMeta_table$sampleID, regexpr("[^\\.]+[^\\.]+", addMeta_table$sampleID))
   print("Reading in meta data table: check if input samples and meta data sampleIDs are identical")
   print(addMeta_table$sampleID)
   stopifnot(sort(addMeta_table$sampleID) == sort(unique(metadata$sampleID)))
 } else if (is.null(opt$add_meta)) {
   print("No input file with additional meta data given.")
   addMeta_table <- NULL
 } else {
   print("Something went wrong reading in the meta data table.")
 }


lookup <-read.csv(opt$lookup, sep = "\t", stringsAsFactors = FALSE)
lookup$Antibody <- sub("_", "-", lookup$Antibody)
print(lookup)
# Read in thresholds
thresholds <- read.csv(opt$thresholds, sep= "\t", stringsAsFactors = FALSE)
thresholds$Antibody<- sub("_", "-", thresholds$Antibody)
rownames(thresholds) <- toupper(thresholds$Antibody)
print(tail(thresholds,20))

set.seed(2)


tmp=as.character(seurat_integrated$celltype_final)
tmp[grep("T.cells.memory.resting",tmp) ]<-"T.cells"
tmp[grep("T.cells.regulatory",tmp) ]<-"T.cells"
tmp[grep("T.cells.memory.activated",tmp) ]<-"T.cells"
tmp[grep("Dendritic.cells.CD",tmp) ]<-"Dendritic.cells"

tmp[grep("Dendritic.cells.AX",tmp) ]<-"Dendritic.cells"
tmp[grep("T.cells.gamma.delta",tmp) ]<-"T.cells"

tmp[grep("T.cells.naive",tmp) ]<-"T.cells.CD4.naive"

tmp[grep("B.cells.naive",tmp) ]<-"B.cells"

tmp[grep("B.cells.memory",tmp) ]<-"B.cells"
seurat_integrated$collapse_celltype <- tmp


seurat_subset=subset(seurat_integrated, subset=celltype_major != "unknown")
seurat_subset=subset(seurat_subset, subset=celltype_major != "uncertain")

config <- read.csv(opt$colour_config, sep = "\t", stringsAsFactors = FALSE)

config$class <- NULL
config$indication <- NULL
# have character vector of colour names from the colour config

config <- rbind(config, c("uncertain", "grey50", "uncertain"))
config <- rbind(config, c("unknown", "black", "unknown"))
# give names to colours from other column of colour config
ct.color <- config$colour
names(ct.color) <- config$mapping
print(ct.color)

# make sure "original objects" contain the correct mapping for the celltypes
names(ct.color)[48]="T.cells.CD4.naive"

# get integer vector containing the index of the matching colour in ct.color for each
# level of seurat_integrated[[]]$celltype_major
id.first.ct <- match(levels(seurat_integrated[[]]$celltype_major), names(ct.color))
id.first.ct
id.final.ct <- match(unique(seurat_subset[[]]$collapse_celltype), names(ct.color))
id.final.ct

colstypes <- c("red2", "green4", "green2", "mediumorchid2", "blue2", "cyan2", "yellow1", "purple", "brown",
               "chocolate1", "chartreuse2", "darkgoldenrod3", "steelblue1", "slateblue3", "olivedrab4", "gold2",
               "violetred3", "darkcyan", "orchid3", "darksalmon", "darkslategrey", "khaki", "indianred2", "magenta", "slategray2",
               "olivedrab1", "mediumaquamarine", "hotpink", "yellow3",
               "bisque4", "darkseagreen1", "dodgerblue3",
               "deeppink4", "sienna4", "mediumorchid4")
###   Plot general cohort composition   ###
print(empty_string)
print("Plot general cohort composition.")
print(Sys.time())

p2 <- DimPlot(seurat_subset, reduction = 'wnn.umap', label = FALSE,group.by = "collapse_celltype",cols = ct.color[id.final.ct],  repel = TRUE, label.size = 2.5) +
  theme(aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(title = "celltype_major", override.aes = list(size = 3))) +
  ggtitle("celltype_major")
p2
ggsave(filename = "Liquid_UMAP_subset.png", width = 30, height = 20, dpi = 300, units = "cm")

p1 <- DimPlot(seurat_integrated, reduction = 'wnn.umap', , group.by = "sample_name", repel = TRUE) +
  theme(aspect.ratio = plot_aspect_ratio)
p1

ggsave(filename="Liquid_UMAP.png", width = 28, height = (22 + 6*0.3), dpi = 300, units = "cm")
