##Script to generate the upset plots. The relationship tables, showing if an antibody is present or not in a sample, have been generated manually 

# Input are tables generated manually 
# Antibody Names have been adapted to contain no spaces & /
# Yes and No has been turned to 0 and 1

#Test of alternative function to better manipulate the vennplot
library(ComplexHeatmap)

input <- read.table("for_6.1C.txt", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
input <- input[,-1]	# Now that AB names are in rownames, I need to remove the column
m <- make_comb_mat(input) # The function needs the table to be in a specific format

m <- m[comb_size(m)>1] # Filter out anything with 0 or 1 overlaps
m <- m[comb_degree(m)>0] # Filter out matches to NO sample
m <- t(m) # Transpose the object, so it has the number of confirmed AB per sample on top


ss = set_size(m)
cs = comb_size(m)
png("6.1C_vennplot_v2.png", width = 700, height = 400, res = 300, units="mm")
ht = UpSet(m,
	column_names_rot = 45,
	column_names_gp = gpar(fontsize =20),
	comb_order = order(comb_size(m), decreasing = TRUE), #Order the matches from larger to smaller
	pt_size = unit(10, "mm"), # Change the size of the dots
	lwd = 4,		# Change the width of the line connecting the dots
	comb_col = "#6FD653",	# Color of the dots and lines
	bg_col = "#DCF5D7",	# Color of the darker background. It does not work on transposed objects
	top_annotation = HeatmapAnnotation(	# Decide what goes in the top annotation
		"Number of confirmed antibodies in sample" = anno_barplot(ss,	# The name given here is both the text written on the plot and the identifier for any next step (e.g. the subsequent call of "decorate_annotation")
			border=FALSE,					# No border around the barplot
			gp = gpar(fill = "grey20", fontsize = 30), 	# Visual parameters for the annotation 
			height=unit(25,"cm"),			# Total height of the annotation
			#width = unit(10,"cm"),
			bar_width = 0.5,			# Width to use for the bars
			axis_param = list(
				labels_rot = 45,
				gp = gpar(fontsize = 30)
			)
		),
		annotation_name_side = "left",	# Where to place the axis name
		annotation_name_rot = 90,	# Rotation of the axis name
		annotation_name_gp = gpar(fontsize = 30, fontface = "bold")
	),
	right_annotation = rowAnnotation(	# Decide what goes in the right annotation
		"Number of antibodies" = anno_barplot(cs,
			baseline =0,		#?
			axis_param = list(	# Rotation of the axis labels
				labels_rot = 0,
				gp = gpar(fontsize = 30)
				),
			border=FALSE,
			gp = gpar(fill = "grey38", fontsize = 30),
			width = unit(20,"cm"),	# Total width of the annotation
		),
		set_name = anno_text(NULL#,	# This parameter is born to manipulate the sample names in non-transposed plots. It still needs to be here and set to NULL for the function to work properly
			#location = 0.5,
			#just = "center",
			#width = max_text_width(set_name(m)) + unit(4, "mm")
		),
		annotation_name_gp = gpar(fontsize = 30)
	),
	left_annotation =NULL,	#Decide what goes in the left annotation
	show_row_names=FALSE
)
ht = draw(ht)	# plot
od = rev(row_order(ht))
decorate_annotation("Number of antibodies", {		# Perform additional beautification
    grid.text(cs[od], y = seq_along(cs), x = unit(cs[od], "native") + unit(20, "pt"), 	# All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 20, col = "black"), rot = 0)
})
od = (column_order(ht))
decorate_annotation("Number of confirmed antibodies in sample", {           # Perform additional beautification
    grid.text(ss[od], x = seq_along(ss), y = unit(ss[od], "native") + unit(20, "pt"),   # All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 30, col = "black"), rot = 0)
})
dev.off()

# Input are tables generated manually 
# Antibody Names have been adapted to contain no spaces & /
# Yes and No has been turned to 0 and 1

#Test of alternative function to better manipulate the vennplot
library(ComplexHeatmap)
input <- read.table("for_4E_mine.txt", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
input <- input[,-1]	# Now that AB names are in rownames, I need to remove the column
m <- make_comb_mat(input) # The function needs the table to be in a specific format

m <- m[comb_size(m)>1] # Filter out anything with 0 or 1 overlaps
m <- m[comb_degree(m)>0] # Filter out matches to NO sample
m <- t(m) # Transpose the object, so it has the number of confirmed AB per sample on top


ss = set_size(m)
cs = comb_size(m)
png("4E_vennplot_v2.png", width = 700, height = 400, res = 300, units="mm")
ht = UpSet(m,
	set_order = c("PBMC_D1_Untreated", "PBMC_D2_Untreated", "PBMC_D3_Untreated", "PBMC_D1_SkinD", "PBMC_D2_SkinD", "PBMC_D3_SkinD", "PBMC_D1_TumorD", "PBMC_D2_TumorD", "PBMC_D3_TumorD"),
	column_names_rot = 45,
	column_names_gp = gpar(fontsize =20),
	comb_order = order(comb_size(m), decreasing = TRUE), #Order the matches from larger to smaller
	pt_size = unit(10, "mm"), # Change the size of the dots
	lwd = 4,		# Change the width of the line connecting the dots
	comb_col = "#6FD653",	# Color of the dots and lines
	bg_col = "#DCF5D7",	# Color of the darker background. It does not work on transposed objects
	top_annotation = HeatmapAnnotation(	# Decide what goes in the top annotation
		"Number of confirmed antibodies in sample" = anno_barplot(ss,	# The name given here is both the text written on the plot and the identifier for any next step (e.g. the subsequent call of "decorate_annotation")
			border=FALSE,					# No border around the barplot
			gp = gpar(fill = "grey20", fontsize = 30), 	# Visual parameters for the annotation 
			height=unit(25,"cm"),			# Total height of the annotation
			#width = unit(10,"cm"),
			bar_width = 0.5,			# Width to use for the bars
			axis_param = list(
				labels_rot = 45,
				gp = gpar(fontsize = 30)
			)
		),
		annotation_name_side = "left",	# Where to place the axis name
		annotation_name_rot = 90,	# Rotation of the axis name
		annotation_name_gp = gpar(fontsize = 30, fontface = "bold")
	),
	right_annotation = rowAnnotation(	# Decide what goes in the right annotation
		"Number of antibodies" = anno_barplot(cs,
			baseline =0,		#?
			axis_param = list(	# Rotation of the axis labels
				labels_rot = 0,
				gp = gpar(fontsize = 30)
				),
			border=FALSE,
			gp = gpar(fill = "grey38", fontsize = 30),
			width = unit(20,"cm"),	# Total width of the annotation
		),
		set_name = anno_text(NULL#,	# This parameter is born to manipulate the sample names in non-transposed plots. It still needs to be here and set to NULL for the function to work properly
			#location = 0.5,
			#just = "center",
			#width = max_text_width(set_name(m)) + unit(4, "mm")
		),
		annotation_name_gp = gpar(fontsize = 30)
	),
	left_annotation =NULL,	#Decide what goes in the left annotation
	show_row_names=FALSE
)
ht = draw(ht)	# plot
od = rev(row_order(ht))
decorate_annotation("Number of antibodies", {		# Perform additional beautification
    grid.text(cs[od], y = seq_along(cs), x = unit(cs[od], "native") + unit(20, "pt"), 	# All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 20, col = "black"), rot = 0)
})
od = (column_order(ht))
decorate_annotation("Number of confirmed antibodies in sample", {           # Perform additional beautification
    grid.text(ss[od], x = seq_along(ss), y = unit(ss[od], "native") + unit(20, "pt"),   # All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 30, col = "black"), rot = 0)
})
dev.off()


# Input are tables generated manually 
# Antibody Names have been adapted to contain no spaces & /
# Yes and No has been turned to 0 and 1

#Test of alternative function to better manipulate the vennplot
library(ComplexHeatmap)
input <- read.table("for_5C.txt", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
input <- input[,-1]	# Now that AB names are in rownames, I need to remove the column
m <- make_comb_mat(input) # The function needs the table to be in a specific format

m <- m[comb_size(m)>1] # Filter out anything with 0 or 1 overlaps
m <- m[comb_degree(m)>0] # Filter out matches to NO sample
m <- t(m) # Transpose the object, so it has the number of confirmed AB per sample on top


ss = set_size(m)
cs = comb_size(m)
png("5C_vennplot_v2.png", width = 700, height = 400, res = 300, units="mm")
ht = UpSet(m,
	column_names_rot = 45,
	column_names_gp = gpar(fontsize =20),
	comb_order = order(comb_size(m), decreasing = TRUE), #Order the matches from larger to smaller
	pt_size = unit(10, "mm"), # Change the size of the dots
	lwd = 4,		# Change the width of the line connecting the dots
	comb_col = "#6FD653",	# Color of the dots and lines
	bg_col = "#DCF5D7",	# Color of the darker background. It does not work on transposed objects
	top_annotation = HeatmapAnnotation(	# Decide what goes in the top annotation
		"Number of confirmed antibodies in sample" = anno_barplot(ss,	# The name given here is both the text written on the plot and the identifier for any next step (e.g. the subsequent call of "decorate_annotation")
			border=FALSE,					# No border around the barplot
			gp = gpar(fill = "grey20", fontsize = 30), 	# Visual parameters for the annotation 
			height=unit(25,"cm"),			# Total height of the annotation
			#width = unit(10,"cm"),
			bar_width = 0.5,			# Width to use for the bars
			axis_param = list(
				labels_rot = 45,
				gp = gpar(fontsize = 30)
			)
		),
		annotation_name_side = "left",	# Where to place the axis name
		annotation_name_rot = 90,	# Rotation of the axis name
		annotation_name_gp = gpar(fontsize = 30, fontface = "bold")
	),
	right_annotation = rowAnnotation(	# Decide what goes in the right annotation
		"Number of antibodies" = anno_barplot(cs,
			baseline =0,		#?
			axis_param = list(	# Rotation of the axis labels
				labels_rot = 0,
				gp = gpar(fontsize = 30)
				),
			border=FALSE,
			gp = gpar(fill = "grey38", fontsize = 30),
			width = unit(20,"cm"),	# Total width of the annotation
		),
		set_name = anno_text(NULL#,	# This parameter is born to manipulate the sample names in non-transposed plots. It still needs to be here and set to NULL for the function to work properly
			#location = 0.5,
			#just = "center",
			#width = max_text_width(set_name(m)) + unit(4, "mm")
		),
		annotation_name_gp = gpar(fontsize = 30)
	),
	left_annotation =NULL,	#Decide what goes in the left annotation
	show_row_names=FALSE
)
ht = draw(ht)	# plot
od = rev(row_order(ht))
decorate_annotation("Number of antibodies", {		# Perform additional beautification
    grid.text(cs[od], y = seq_along(cs), x = unit(cs[od], "native") + unit(20, "pt"), 	# All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 20, col = "black"), rot = 0)
})
od = (column_order(ht))
decorate_annotation("Number of confirmed antibodies in sample", {           # Perform additional beautification
    grid.text(ss[od], x = seq_along(ss), y = unit(ss[od], "native") + unit(20, "pt"),   # All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 30, col = "black"), rot = 0)
})
dev.off()


# Input are tables generated manually 
# Antibody Names have been adapted to contain no spaces & /
# Yes and No has been turned to 0 and 1

#Test of alternative function to better manipulate the vennplot
library(ComplexHeatmap)
input <- read.table("for_5F_v2.txt", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
input <- input[,-1]	# Now that AB names are in rownames, I need to remove the column
m <- make_comb_mat(input) # The function needs the table to be in a specific format

m <- m[comb_size(m)>1] # Filter out anything with 0 or 1 overlaps
m <- m[comb_degree(m)>0] # Filter out matches to NO sample
m <- t(m) # Transpose the object, so it has the number of confirmed AB per sample on top


ss = set_size(m)
cs = comb_size(m)
png("5F_vennplot_v2.png", width = 700, height = 400, res = 300, units="mm")
ht = UpSet(m,
	column_names_rot = 45,
	column_names_gp = gpar(fontsize =20),
	comb_order = order(comb_size(m), decreasing = TRUE), #Order the matches from larger to smaller
	pt_size = unit(10, "mm"), # Change the size of the dots
	lwd = 4,		# Change the width of the line connecting the dots
	comb_col = "#6FD653",	# Color of the dots and lines
	bg_col = "#DCF5D7",	# Color of the darker background. It does not work on transposed objects
	top_annotation = HeatmapAnnotation(	# Decide what goes in the top annotation
		"Number of confirmed antibodies in sample" = anno_barplot(ss,	# The name given here is both the text written on the plot and the identifier for any next step (e.g. the subsequent call of "decorate_annotation")
			border=FALSE,					# No border around the barplot
			gp = gpar(fill = "grey20", fontsize = 30), 	# Visual parameters for the annotation 
			height=unit(25,"cm"),			# Total height of the annotation
			#width = unit(10,"cm"),
			bar_width = 0.5,			# Width to use for the bars
			axis_param = list(
				labels_rot = 45,
				gp = gpar(fontsize = 30)
			)
		),
		annotation_name_side = "left",	# Where to place the axis name
		annotation_name_rot = 90,	# Rotation of the axis name
		annotation_name_gp = gpar(fontsize = 30, fontface = "bold")
	),
	right_annotation = rowAnnotation(	# Decide what goes in the right annotation
		"Number of antibodies" = anno_barplot(cs,
			baseline =0,		#?
			axis_param = list(	# Rotation of the axis labels
				labels_rot = 0,
				gp = gpar(fontsize = 30)
				),
			border=FALSE,
			gp = gpar(fill = "grey38", fontsize = 30),
			width = unit(20,"cm"),	# Total width of the annotation
		),
		set_name = anno_text(NULL#,	# This parameter is born to manipulate the sample names in non-transposed plots. It still needs to be here and set to NULL for the function to work properly
			#location = 0.5,
			#just = "center",
			#width = max_text_width(set_name(m)) + unit(4, "mm")
		),
		annotation_name_gp = gpar(fontsize = 30)
	),
	left_annotation =NULL,	#Decide what goes in the left annotation
	show_row_names=FALSE
)
ht = draw(ht)	# plot
od = rev(row_order(ht))
decorate_annotation("Number of antibodies", {		# Perform additional beautification
    grid.text(cs[od], y = seq_along(cs), x = unit(cs[od], "native") + unit(20, "pt"), 	# All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 20, col = "black"), rot = 0)
})
od = (column_order(ht))
decorate_annotation("Number of confirmed antibodies in sample", {           # Perform additional beautification
    grid.text(ss[od], x = seq_along(ss), y = unit(ss[od], "native") + unit(20, "pt"),   # All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 30, col = "black"), rot = 0)
})
dev.off()


# Input are tables generated manually 
# Antibody Names have been adapted to contain no spaces & /
# Yes and No has been turned to 0 and 1

#Test of alternative function to better manipulate the vennplot
library(ComplexHeatmap)
input <- read.table("for_S5C_v2.txt", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
input <- input[,-1]	# Now that AB names are in rownames, I need to remove the column
m <- make_comb_mat(input) # The function needs the table to be in a specific format

m <- m[comb_size(m)>1] # Filter out anything with 0 or 1 overlaps
m <- m[comb_degree(m)>0] # Filter out matches to NO sample
m <- t(m) # Transpose the object, so it has the number of confirmed AB per sample on top


ss = set_size(m)
cs = comb_size(m)
png("S5C_vennplot_v2.png", width = 700, height = 400, res = 300, units="mm")
ht = UpSet(m,
	column_names_rot = 45,
	column_names_gp = gpar(fontsize =20),
	comb_order = order(comb_size(m), decreasing = TRUE), #Order the matches from larger to smaller
	pt_size = unit(10, "mm"), # Change the size of the dots
	lwd = 4,		# Change the width of the line connecting the dots
	comb_col = "#6FD653",	# Color of the dots and lines
	bg_col = "#DCF5D7",	# Color of the darker background. It does not work on transposed objects
	top_annotation = HeatmapAnnotation(	# Decide what goes in the top annotation
		"Number of confirmed antibodies in sample" = anno_barplot(ss,	# The name given here is both the text written on the plot and the identifier for any next step (e.g. the subsequent call of "decorate_annotation")
			border=FALSE,					# No border around the barplot
			gp = gpar(fill = "grey20", fontsize = 30), 	# Visual parameters for the annotation 
			height=unit(25,"cm"),			# Total height of the annotation
			#width = unit(10,"cm"),
			bar_width = 0.5,			# Width to use for the bars
			axis_param = list(
				labels_rot = 45,
				gp = gpar(fontsize = 30)
			)
		),
		annotation_name_side = "left",	# Where to place the axis name
		annotation_name_rot = 90,	# Rotation of the axis name
		annotation_name_gp = gpar(fontsize = 30, fontface = "bold")
	),
	right_annotation = rowAnnotation(	# Decide what goes in the right annotation
		"Number of antibodies" = anno_barplot(cs,
			baseline =0,		#?
			axis_param = list(	# Rotation of the axis labels
				labels_rot = 0,
				gp = gpar(fontsize = 30)
				),
			border=FALSE,
			gp = gpar(fill = "grey38", fontsize = 30),
			width = unit(20,"cm"),	# Total width of the annotation
		),
		set_name = anno_text(NULL#,	# This parameter is born to manipulate the sample names in non-transposed plots. It still needs to be here and set to NULL for the function to work properly
			#location = 0.5,
			#just = "center",
			#width = max_text_width(set_name(m)) + unit(4, "mm")
		),
		annotation_name_gp = gpar(fontsize = 30)
	),
	left_annotation =NULL,	#Decide what goes in the left annotation
	show_row_names=FALSE
)
ht = draw(ht)	# plot
od = rev(row_order(ht))
decorate_annotation("Number of antibodies", {		# Perform additional beautification
    grid.text(cs[od], y = seq_along(cs), x = unit(cs[od], "native") + unit(20, "pt"), 	# All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 20, col = "black"), rot = 0)
})
od = (column_order(ht))
decorate_annotation("Number of confirmed antibodies in sample", {           # Perform additional beautification
    grid.text(ss[od], x = seq_along(ss), y = unit(ss[od], "native") + unit(20, "pt"),   # All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 30, col = "black"), rot = 0)
})
dev.off()


# Input are tables generated manually 
# Antibody Names have been adapted to contain no spaces & /
# Yes and No has been turned to 0 and 1

#Test of alternative function to better manipulate the vennplot
library(ComplexHeatmap)
input <- read.table("for_S5F.txt", sep="\t", header = TRUE)
head(input)
rownames(input) <- input$Antibody
input <- input[,-1]	# Now that AB names are in rownames, I need to remove the column
m <- make_comb_mat(input) # The function needs the table to be in a specific format

m <- m[comb_size(m)>1] # Filter out anything with 0 or 1 overlaps
m <- m[comb_degree(m)>0] # Filter out matches to NO sample
m <- t(m) # Transpose the object, so it has the number of confirmed AB per sample on top


ss = set_size(m)
cs = comb_size(m)
png("S5F_vennplot_v2.png", width = 700, height = 400, res = 300, units="mm")
ht = UpSet(m,
	column_names_rot = 45,
	column_names_gp = gpar(fontsize =20),
	comb_order = order(comb_size(m), decreasing = TRUE), #Order the matches from larger to smaller
	pt_size = unit(10, "mm"), # Change the size of the dots
	lwd = 4,		# Change the width of the line connecting the dots
	comb_col = "#6FD653",	# Color of the dots and lines
	bg_col = "#DCF5D7",	# Color of the darker background. It does not work on transposed objects
	top_annotation = HeatmapAnnotation(	# Decide what goes in the top annotation
		"Number of confirmed antibodies in sample" = anno_barplot(ss,	# The name given here is both the text written on the plot and the identifier for any next step (e.g. the subsequent call of "decorate_annotation")
			border=FALSE,					# No border around the barplot
			gp = gpar(fill = "grey20", fontsize = 30), 	# Visual parameters for the annotation 
			height=unit(25,"cm"),			# Total height of the annotation
			#width = unit(10,"cm"),
			bar_width = 0.5,			# Width to use for the bars
			axis_param = list(
				labels_rot = 45,
				gp = gpar(fontsize = 30)
			)
		),
		annotation_name_side = "left",	# Where to place the axis name
		annotation_name_rot = 90,	# Rotation of the axis name
		annotation_name_gp = gpar(fontsize = 30, fontface = "bold")
	),
	right_annotation = rowAnnotation(	# Decide what goes in the right annotation
		"Number of antibodies" = anno_barplot(cs,
			baseline =0,		#?
			axis_param = list(	# Rotation of the axis labels
				labels_rot = 0,
				gp = gpar(fontsize = 30)
				),
			border=FALSE,
			gp = gpar(fill = "grey38", fontsize = 30),
			width = unit(20,"cm"),	# Total width of the annotation
		),
		set_name = anno_text(NULL#,	# This parameter is born to manipulate the sample names in non-transposed plots. It still needs to be here and set to NULL for the function to work properly
			#location = 0.5,
			#just = "center",
			#width = max_text_width(set_name(m)) + unit(4, "mm")
		),
		annotation_name_gp = gpar(fontsize = 30)
	),
	left_annotation =NULL,	#Decide what goes in the left annotation
	show_row_names=FALSE
)
ht = draw(ht)	# plot
od = rev(row_order(ht))
decorate_annotation("Number of antibodies", {		# Perform additional beautification
    grid.text(cs[od], y = seq_along(cs), x = unit(cs[od], "native") + unit(20, "pt"), 	# All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 20, col = "black"), rot = 0)
})
od = (column_order(ht))
decorate_annotation("Number of confirmed antibodies in sample", {           # Perform additional beautification
    grid.text(ss[od], x = seq_along(ss), y = unit(ss[od], "native") + unit(20, "pt"),   # All this section adds the number of ABs on top of the bars in the right annotation
        default.units = "native", just = "center", 
        gp = gpar(fontsize = 30, col = "black"), rot = 0)
})
dev.off()



