library(optparse)

option_list = list(
make_option("--seurat", type = "character", default = "solid.integrated_seurat_object.RDS"),
make_option("--outname", type = "character"),
make_option("--lookup", type = "character", default = "reference_lookup.tab"))

library(Seurat)
library(SingleCellExperiment)
seurat_integrated <- readRDS(opt$seurat)
rna_counts <- as.data.frame(seurat_integrated@assays$RNA@counts)
rna_count_sums <- rowSums(rna_counts)
rna_zero_counts <- names(rna_count_sums)[which(rna_count_sums==0)]
message(paste0("Found ", nrow(seurat_integrated), " features.\nFound ", length(rna_zero_counts), " features with no counts"))
adt_counts <- as.data.frame(seurat_integrated@assays$adt@counts)
adt_count_sums <- rowSums(adt_counts)
adt_zero_counts <- names(adt_count_sums)[which(adt_count_sums==0)]
message(paste0("Found ", nrow(seurat_integrated), " features.\nFound ", length(adt_zero_counts), " features with no counts"))



library(Hmisc)
library(dplyr)
library(ggplot2)

lookup <- read.delim(opt$lookup)
lookup[which(lookup[,2]=="CD4_RPA-T4"),2] <- "CD4-RPA-T4"
lookup[which(lookup[,2]=="CD8a_RPA-T8"),2] <- "CD8a-RPA-T8"
lookup[which(lookup[,2]=="TCR-a_b"),2] <- "TCR-a-b"


count_table <- t(rbind(rna_counts, adt_counts))

lookup_tmp <- (lookup)
lookup_tmp[,2] <- paste0("ADT-", lookup[,2])
lookup_tmp <- unlist(lookup_tmp)
cols_to_keep <- which(colnames(count_table) %in% lookup_tmp)
count_table <- count_table[,cols_to_keep]

correlation_table <- rcorr(count_table, type="pearson")

correlation_df <- NULL
pval_df <- NULL
tmp_corr_df <- NULL
tmp_pval_df <- NULL

message("looping")
for(i in 1:nrow(correlation_table$r)){
  current_gene <- rownames(correlation_table$r)[i]
  if(length(grep("^ADT-", current_gene))>0){
    next()
  }
  lookup_row <- which(lookup[,1] == current_gene)
  if(length(lookup_row) == 0){next()}
  for(j in 1:length(lookup_row)){
  lookup_adt <- paste0("ADT-",lookup[lookup_row[j],2])
  current_adt_col <- which(colnames(correlation_table$r) == lookup_adt)

  tmp_corr_df <- rbind(tmp_corr_df, data.frame(tmpname= correlation_table$r[i, current_adt_col]) )
  rownames(tmp_corr_df)[nrow(tmp_corr_df)] <- paste0(current_gene, "_", lookup_adt)
  tmp_pval_df <- rbind(tmp_pval_df, data.frame(tmpname= correlation_table$P[i, current_adt_col]) )
  rownames(tmp_pval_df)[nrow(tmp_pval_df)] <- paste0(current_gene, "_", lookup_adt)
}}
correlation_df <- tmp_corr_df
pval_df <- tmp_pval_df

saveRDS(correlation_df, "correlation_df.RDS")
saveRDS(pval_df, "pval_df.RDS")
 tmp=cbind(correlation_df, pval_df)
colnames(tmp) <- c("r", "P")
tmp=tmp[!is.nan(tmp[,1]),]
tmp=tmp[!is.na(tmp[,1]),]
write.table(tmp, paste0(opt$outname, "_correlation_table.tsv"), sep="\t", quote=F)
tmp[which(tmp[,2]==0),2]<-min(tmp[-(which(tmp[,2]==0)),2])

p=ggplot(data=tmp, aes(x=reorder(rownames(tmp), P), y=r)) + geom_bar(stat="identity", aes(fill=-log10(P))) + theme(axis.text.x =element_text(angle=90,hjust=1,vjust=0.5)) + scale_fill_gradient2(midpoint=-log10(0.05), low="grey", mid="blue", high="red") + xlab("") + ylab("Correlation coefficient r") + labs(fill="-log10(PValue)") + ggtitle(paste0(opt$outname, " - Gene-protein expression correlation")) #+ geom_vline(xintercept=86.5, linetype="dotted", color = "red", size = 1.5) 
ggsave(plot=p, filename=paste0(opt$outname, "_correlation_plot_005_pvalsort_final.pdf"), width=20 )

p=ggplot(data=tmp, aes(x=reorder(rownames(tmp), r), y=r)) + geom_bar(stat="identity", aes(fill=-log10(P))) + theme(axis.text.x =element_text(angle=90,hjust=1,vjust=0.5)) + scale_fill_gradient2(midpoint=-log10(0.05), low="grey", mid="blue", high="red") + xlab("") + ylab("Correlation coefficient r") + labs(fill="-log10(PValue)") + ggtitle(paste0(opt$outname, " - Gene-protein expression correlation"))# + geom_vline(xintercept=12.5, linetype="dotted", color = "red", size = 1.5) + geom_vline(xintercept=6.5, linetype="dotted", color = "red", size = 1.5)
ggsave(plot=p, filename=paste0(opt$outname,"_correlation_plot_005_rsort_final.pdf"), width=20 )



