# Script to extract the expression levels of GEX and adt from an RDS containing an integrated Seurat object
# The script also structures this information in a manner useful to build a correlation matrix and then plots
# an heatmap of the gene/adt correlation

library(Seurat)
library(SingleCellExperiment)
library(optparse)

option_list = list(
make_option("--seurat_object", type = "character", default = "solid.integrated_seurat_object.RDS"),
make_option("--samples", type = "character", help = "list of samples, separated by comman"))
opt_parser = OptionParser(option_list = option_list)
opt = parse_args(opt_parser)

samples = unlist(strsplit(opt$samples,","))

for(mysample in samples){
  message(mysample)
  seurat_subset <- subset(seurat_integrated, subset = sample_name == mysample)
  saveRDS(seurat_subset, paste0(mysample, ".RDS"))
  tmp = as.SingleCellExperiment(seurat_subset, assay="RNA")
  saveRDS(tmp, paste0(mysample, "RNA_SCE.RDS"))
  tmp = as.SingleCellExperiment(seurat_subset, assay="adt")
  saveRDS(tmp, paste0(mysample, "adt_SCE.RDS"))
  tmp = as.SingleCellExperiment(seurat_subset, assay="integratedGEX")
  saveRDS(tmp, paste0(mysample, "integratedGEX_SCE.RDS"))
  tmp = as.SingleCellExperiment(seurat_subset, assay="integratedADT")
  saveRDS(tmp, paste0(mysample, "integratedADT_SCE.RDS"))

}
